#!/usr/bin/env ruby

require 'test/unit'
$:.unshift File.join(File.dirname(__FILE__), "..", "lib")
require 'WiKID'
require 'SecureRandom'
require 'rexml/document'
include REXML

TEST_OPTIONS_FILE = File.join(File.dirname(__FILE__), "..", "test-options.rb")
config_file_exists = FileTest.exists?(TEST_OPTIONS_FILE)
if config_file_exists
  require TEST_OPTIONS_FILE
end

SERVER_HOST = "127.0.0.1" if !defined?(SERVER_HOST)
DOMAIN_CODE = "127000000001" if !defined?(DOMAIN_CODE)
SERVER_PORT = 8388 if !defined?(SERVER_PORT)
CLIENT_KEY_FILE = '' if !defined?(CLIENT_KEY_FILE)
CLIENT_KEY_PASS = '' if !defined?(CLIENT_KEY_PASS)

puts "* Running with server: #{SERVER_HOST}:#{SERVER_PORT}"

class TestWiKID < Test::Unit::TestCase

  def test_init_module

    wc = WiKID::Auth.new(SERVER_HOST, SERVER_PORT, CLIENT_KEY_FILE, CLIENT_KEY_PASS)

    assert_not_nil(wc, "WiKID module successfully loaded!")

  end

  def test_check_keys

    wc = WiKID::Auth.new(SERVER_HOST, SERVER_PORT, CLIENT_KEY_FILE, CLIENT_KEY_PASS)
    assert_nothing_raised do

      wc.checkKeys()
    end
  end

  def test_invalid_user_registration

    username = 'test'
    domaincode = DOMAIN_CODE

    wc = WiKID::Auth.new(SERVER_HOST, SERVER_PORT, CLIENT_KEY_FILE, CLIENT_KEY_PASS)

    regcode = ""
    result = wc.registerUsername(username, regcode, domaincode)
    assert_not_equal(0, result, "Properly failed to register user")

  end

  def test_valid_user_registration

    username = 'test-' + SecureRandom.uuid
    domaincode = DOMAIN_CODE

    wc = WiKID::Auth.new(SERVER_HOST, SERVER_PORT, CLIENT_KEY_FILE, CLIENT_KEY_PASS)

    output=`#{JCMD_REGISTER} #{DOMAIN_CODE}`
    matches = /code=(\w+)/.match(output)
    regcode = matches[1] || ''
    result = wc.registerUsername(username, regcode, DOMAIN_CODE)

    assert_equal(0, result, "Failed to register user")

    # cleanup
    wc.deleteUser(username, domaincode)

  end

  def test_invalid_pin

  end

  def test_valid_pin

  end

  def test_find_user

    username = 'test-' + SecureRandom.uuid
    domaincode = DOMAIN_CODE

    wc = WiKID::Auth.new(SERVER_HOST, SERVER_PORT, CLIENT_KEY_FILE, CLIENT_KEY_PASS)

    output=`#{JCMD_REGISTER} #{DOMAIN_CODE}`
    matches = /code=(\w+)/.match(output)
    regcode = matches[1] || ''
    result = wc.registerUsername(username, regcode, DOMAIN_CODE)

    user_xml = wc.findUser(username, domaincode)

    user_id = XPath.first(user_xml, '//user-id/text()')

    assert_equal(user_id, username, "Failed to lookup user")

    # cleanup
    wc.deleteUser(username, domaincode)

  end


  def test_update_user

    username = 'test-' + SecureRandom.uuid
    domaincode = DOMAIN_CODE

    wc = WiKID::Auth.new(SERVER_HOST, SERVER_PORT, CLIENT_KEY_FILE, CLIENT_KEY_PASS)

    output=`#{JCMD_REGISTER} #{DOMAIN_CODE}`
    matches = /code=(\w+)/.match(output)
    regcode = matches[1] || ''
    wc.registerUsername(username, regcode, DOMAIN_CODE)

    ## get the XML string to modify with
    user_xml = wc.findUser(username, domaincode)

    ## 1. modify the number of bad attempts
    attempts = XPath.first(user_xml, '//bad-passcode-attempts/text()')
    assert_equal(0, attempts, "Number of bad attempts is aleady non-zero!")

    user_xml.elements['bad-passcode-attempts'].text = 2

    result = wc.updateUser(username, domaincode, user_xml)
    assert_equal(true, result, "Failed to modify the number of bad attempts!")

    user_xml = wc.findUser(username, domaincode)
    attempts = XPath.first(user_xml, '//bad-passcode-attempts/text()')
    assert_equal(2, attempts, "Failed to correctly set the the number of bad attempts!")

    ## 2. modify the user status
    status = XPath.first(user_xml, '//status/text()')
    assert_equal(1, status, "Status is already disabled!")

    user_xml.elements['status'].text = 0

    result = wc.updateUser(username, domaincode, user_xml)
    assert_equal(true, result, "Failed to modify the status!")

    user_xml = wc.findUser(username, domaincode)
    status = XPath.first(user_xml, '//status/text()')
    assert_equal(0, status, "Failed to modify the status value!")

    ## 3. modify the user note
    note = XPath.first(user_xml, '//note/text()')
    assert_equal('', note.to_s, "Note is already non-empty!")

    new_note_text = "This is a test note"
    user_xml.elements['note'].text = new_note_text

    result = wc.updateUser(username, domaincode, user_xml)
    assert_equal(true, result, "Failed to modify the note!")

    user_xml = wc.findUser(username, domaincode)
    note = XPath.first(user_xml, '//note/text()')
    assert_equal(new_note_text, note.to_s, "Failed to modity the note value!")

    # cleanup
    wc.deleteUser(username, domaincode)

  end

  def test_delete_unknown_user

    username = 'junk-test-' + SecureRandom.uuid
    domaincode = DOMAIN_CODE

    wc = WiKID::Auth.new(SERVER_HOST, SERVER_PORT, CLIENT_KEY_FILE, CLIENT_KEY_PASS)

    result = wc.deleteUser(username, domaincode)

    assert_equal(false, result, "Unable to delete unknown user")

    # cleanup
    wc.deleteUser(username, domaincode)

  end

  def test_delete_user

    username = 'test-' + SecureRandom.uuid
    domaincode = DOMAIN_CODE

    wc = WiKID::Auth.new(SERVER_HOST, SERVER_PORT, CLIENT_KEY_FILE, CLIENT_KEY_PASS)

    output=`#{JCMD_REGISTER} #{DOMAIN_CODE}`
    matches = /code=(\w+)/.match(output)
    regcode = matches[1] || ''
    wc.registerUsername(username, regcode, DOMAIN_CODE)

    result = wc.deleteUser(username, domaincode)

    assert_equal(true, result, "Failed to delete user")

    user_xml = wc.findUser(username, domaincode)

    assert_nil(user_xml, "User not deleted")

  end


end
